@@ -61,7 +61,7 @@ class PGPMIME(KryptoMIME):
     def _fix_quoting(text):
         "fix broken Thunderbird quoting"
         import re
-        text = re.sub('^=3D','=',text,flags=re.MULTILINE)
+        text = re.sub(re.compile('^=3D',flags=re.MULTILINE),'=',text)
         return text
 
     @staticmethod
@@ -309,12 +309,17 @@ class PGPMIME(KryptoMIME):
         from email.message import Message
         import email.utils
         if type(mail)==str: mail = Parser().parsestr(mail)
-        elif not isinstance(mail,Message): return False, None
+        elif not isinstance(mail,Message): return False, results
         sender = mail.get('from', [])
         sender = self.find_key(email.utils.parseaddr(sender)[1])
         ciphertext, is_pgpmime = self._ciphertext(mail)
         if ciphertext: # Ciphertext present?
             results['encrypted'] = True
+            if not 'default_key' in kwargs:
+                if not self.default_key: return False, results # key required
+                kwargs['default_key'] = True
+            elif kwargs['default_key']==False or (kwargs['default_key']==True and not self.default_key):
+                return False, results # key required
             ciphertext = self._fix_quoting(ciphertext)
             result = self.decrypt_str(ciphertext, **kwargs)
             results['decryption'] = result
@@ -359,12 +364,17 @@ class PGPMIME(KryptoMIME):
         from email.message import Message
         import email.utils
         if type(mail)==str: mail = Parser().parsestr(mail)
-        elif not isinstance(mail,Message): return None, False, None
+        elif not isinstance(mail,Message): return None, False, results
         sender = mail.get('from', [])
         sender = self.find_key(email.utils.parseaddr(sender)[1])
         ciphertext, is_pgpmime = self._ciphertext(mail)
         if ciphertext: # Ciphertext present? Decode
             results['encrypted'] = True
+            if not 'default_key' in kwargs:
+                if not self.default_key: return None, False, results # key required
+                kwargs['default_key'] = True
+            elif kwargs['default_key']==False or (kwargs['default_key']==True and not self.default_key):
+                return None, False, results # key required
             ciphertext = self._fix_quoting(ciphertext)
             result = self.decrypt_str(ciphertext, **kwargs)
             results['decryption'] = result
@@ -404,16 +414,21 @@ class PGPMIME(KryptoMIME):
         if not isinstance(mail,(Message,str)): return None, None
         mail = protect_mail(mail,ending='\r\n',sevenbit=True) # fix line endings + 7bit
         if not 'default_key' in kwargs:
+            if not self.default_key: return None, None # key required
             import email.utils
             sender = mail.get('from', [])
             sender = email.utils.parseaddr(sender)[1]
             kwargs['default_key'] = sender
+        elif kwargs['default_key']==False or (kwargs['default_key']==True and not self.default_key):
+            return None, None # key required
         plaintext, submsg = self._plaintext(mail, inline)
         # Generate signature, report errors
-        if not mail.is_multipart() and inline:
-            result = self.sign_str(plaintext, clearsign=True, detach=False, **kwargs)
-        else:
-            result = self.sign_str(plaintext, clearsign=False, detach=True, **kwargs)
+        try:
+            if not mail.is_multipart() and inline:
+                result = self.sign_str(plaintext, clearsign=True, detach=False, **kwargs)
+            else:
+                result = self.sign_str(plaintext, clearsign=False, detach=True, **kwargs)
+        except: result = None
         if not result: return None, result
         signature = str(result)
         # Compile signed message
@@ -476,12 +491,19 @@ class PGPMIME(KryptoMIME):
         ccs = mail.get_all('cc', [])
         recipients = [email.utils.formataddr(to) for to in email.utils.getaddresses(tos + ccs)]
         if toself and not sender in recipients: recipients.append(sender)
-        if sign and not 'default_key' in kwargs: kwargs['default_key'] = sender
+        if sign:
+            if not 'default_key' in kwargs:
+                if not self.default_key: return None, None # key required
+                kwargs['default_key'] = sender
+            elif kwargs['default_key']==False or (kwargs['default_key']==True and not self.default_key):
+                return None, None # key required
         plaintext, submsg = self._plaintext(mail, inline, protect=False)
         # Do encryption, report errors
         kwargs['sign'] = sign
         kwargs['armor'] = True
-        result = self.encrypt_str(plaintext, recipients, **kwargs)
+        try:
+            result = self.encrypt_str(plaintext, recipients, **kwargs)
+        except: result = None
         if not result: return None, result
         ciphertext = str(result)
         if verify:
@@ -550,19 +572,23 @@ class GPGMIME(PGPMIME):
         super(GPGMIME,self).__init__(default_key)
 
     def _set_default_key(self,kwargs):
-        if self.default_key and 'default_key' in kwargs:
-            key = kwargs['default_key']
-            if not key:
-                del kwargs['default_key']
-                return kwargs
+        if not 'default_key' in kwargs: return kwargs
+        key = kwargs['default_key']
+        if not key:
+            del kwargs['default_key']
+            return kwargs
+        if key!=True:
+            kwargs['default_key'] = key = self.find_key(key,secret=True)
+            assert key, "sender key missing"
+        else: assert self.default_key, "default key missing"
+        if self.default_key:
             if type(self.default_key) in (tuple,list):
                 defkey,passphrase = self.default_key
             else:
                 defkey,passphrase = self.default_key,None
             if key!=True:
-                key = self.find_key(key,secret=True)
                 if key != defkey: return kwargs
-            kwargs['default_key'] = defkey
+            else: kwargs['default_key'] = defkey
             if not 'passphrase' in kwargs and passphrase:
                 kwargs['passphrase'] = passphrase
         return kwargs
@@ -590,11 +616,11 @@ class GPGMIME(PGPMIME):
         import gnupg
         if signature:
             import os, tempfile
-            tmp = tempfile.NamedTemporaryFile(prefix='gnupg',delete=False)
+            tmp = tempfile.NamedTemporaryFile(mode='w+',prefix='gnupg',delete=False)
             fd, f = tmp.file, tmp.name
             fd.write(data)
             fd.close()
-            tmp = tempfile.NamedTemporaryFile(prefix='gnupg',delete=False)
+            tmp = tempfile.NamedTemporaryFile(mode='w+',prefix='gnupg',delete=False)
             fd, fn = tmp.file, tmp.name
             fd.write(signature)
             fd.close()
@@ -620,6 +646,18 @@ class GPGMIME(PGPMIME):
             text += line
         return text
 
+    def pubkey_attachment(self,key):
+        "returns an attachment with the specified public key"
+        from email.mime.text import MIMEText
+        key = self.find_key(key) or key
+        pubkey = self.gpg.export_keys(key)
+        attach = MIMEText(pubkey)
+        attach.set_type("application/pgp-keys")
+        fname= key+'.asc'
+        attach.set_param('name',fname)
+        attach.add_header('Content-Disposition', 'attachment', filename=fname)
+        return attach
+
     def _encrypt_params(self, recipients, kwargs):
         if 'sign' in kwargs:
             if kwargs['sign']: kwargs['default_key'] = kwargs['sign']
